#include <Ogre.h>
#include <OgreErrorDialog.h>

using namespace Ogre;
using namespace std;

int main(int, char **)
{
  try
  {
    //////////////////////////////////////////////////////////////
    // Initialization of Root and RenderWindow
    //////////////////////////////////////////////////////////////
    Root* mRoot = new Root("../../plugins.cfg","../ogre.cfg","ogre.log");
    if (!mRoot->restoreConfig())
    {
      if (!mRoot->showConfigDialog())
      {
        return 1;
      }
    }
    RenderWindow* mWindow = mRoot->initialise(true,"Application");
    //////////////////////////////////////////////////////////////
    // Initialization of Scene
    //////////////////////////////////////////////////////////////
    SceneManager* mSceneManager = mRoot->createSceneManager(ST_GENERIC,"sceneManager");
    Camera* mCamera = mSceneManager->createCamera("Camera");
    mCamera->setPosition(Vector3(0.0f,0.0f,500.0f));
    mCamera->lookAt(Vector3(0.0f,0.0f,0.0f));
    mCamera->setNearClipDistance(5.0f);
    mCamera->setFarClipDistance(5000.0f);
    ////////////////////////////////////////////////////////////
    // Binding of Scene and RenderWindow
    ////////////////////////////////////////////////////////////
    Viewport* mViewport = mWindow->addViewport(mCamera);
    mViewport->setBackgroundColour(ColourValue(0.0f,0.0f,0.0f));

    mCamera->setAspectRatio(Real(mViewport->getActualWidth()) /
                            Real(mViewport->getActualHeight()));
    //////////////////////////////////////////////////////////////
    // Running the main loop
    //////////////////////////////////////////////////////////////
    mRoot->startRendering();

    delete mRoot;
  }
  catch (Exception& e)
  {
    ErrorDialog* dlg = PlatformManager::getSingleton().createErrorDialog();
    dlg->display(e.getFullDescription());
    delete dlg;
    return 1;
  }
  return 0;
}
