#include "ogreapplication.h"
#include <OgreErrorDialog.h>

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Interface of MyApplication
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class MyApplication : public OgreApplication
{
public:
  MyApplication();
  virtual void init();
};

using namespace Ogre;
using namespace std;

//************************************************************
// Implementation of MyApplication
//************************************************************
MyApplication::MyApplication()
  : OgreApplication("My Application")
{
}
void
MyApplication::init()
{
  mSceneManager->setAmbientLight(ColourValue(0.8f,0.8f,0.8f));

  // Adds a ninja      
  Entity *ninja = mSceneManager->createEntity("Ninja","ninja.mesh");
  SceneNode* ninjaNode = mSceneManager->getRootSceneNode()->createChildSceneNode("NinjaNode");
  ninjaNode->attachObject(ninja);
  Real r = ninja->getBoundingRadius();
  // Overidde the default material for this instance of the mesh
  ninja->getSubEntity(0)->setMaterialName("Tutorial4/InflateBody");
  ninja->getSubEntity(1)->setMaterialName("Tutorial4/InflateSaber");

  // Adds a ground  
  MeshManager::getSingleton().createPlane("plane.mesh","Custom",Plane(Vector3::UNIT_Z,Vector3::ZERO),2*r,2*r,20,20);
  Entity* ground = mSceneManager->createEntity("Ground","plane.mesh");
  ground->setMaterialName("Examples/Rocky");
  SceneNode* groundNode = mSceneManager->getRootSceneNode()->createChildSceneNode();
  groundNode->attachObject(ground);
  groundNode->setOrientation(Quaternion(Radian(-Math::HALF_PI),Vector3::UNIT_X));
  
  // Adds a light 
  Light* light = mSceneManager->createLight("Light1"); 
  light->setType(Light::LT_SPOTLIGHT);
  light->setDiffuseColour(ColourValue(0.25f,0.25f,0.0f));
  light->setSpecularColour(ColourValue(0.25f,0.25f,0.0f));
  light->setAttenuation(8000,1,0.0005,0);
  light->setSpotlightRange(Ogre::Degree(60), Ogre::Degree(70));
  light->setDirection(Vector3::NEGATIVE_UNIT_Y);

  SceneNode* lightNode = ninjaNode->createChildSceneNode("LightNode");
  lightNode->attachObject(light);
  Real y = 1.2f*r;
  lightNode->setPosition(0,y,0);

  BillboardSet* lightbillboardset = mSceneManager->createBillboardSet("lightbbs", 1);
  lightbillboardset->setMaterialName("Examples/Flare");
  Billboard* lightbillboard = lightbillboardset->createBillboard(0,0,0,ColourValue(0.5,0.3,0.0f));
  lightNode->attachObject(lightbillboardset);
  
  // Animating the light
  Animation* animation = mSceneManager->createAnimation("LightAnim",10.0f);
  animation->setInterpolationMode(Animation::IM_SPLINE);
  AnimationTrack* track = animation->createTrack(0,lightNode);

  Real x = 0.6f*r;
  Real z = 0.6f*r;
  KeyFrame* key;
  key = track->createKeyFrame(0.0f);
  key->setTranslate(Vector3(-x, y,-z));
  key = track->createKeyFrame(2.5f);
  key->setTranslate(Vector3( x, y,-z));
  key = track->createKeyFrame(5.0f);
  key ->setTranslate(Vector3( x, y, z));
  key = track->createKeyFrame(7.5f);
  key->setTranslate(Vector3(-x, y, z));
  key = track->createKeyFrame(10.0f);
  key->setTranslate(Vector3(-x, y,-z));
  manage(mSceneManager->createAnimationState("LightAnim"),true,true);
  lightNode->setAutoTracking(true,ninjaNode,Vector3::NEGATIVE_UNIT_Y);

  // Animate the ninja
  manage(ninja->getAnimationState("Idle3"),true,true);
  // Add a skybox
  mSceneManager->setSkyBox(true,"Examples/StormySkyBox",50);

}
int main(int, char **)
{
  MyApplication application;
  try
  {
    application.addResourceLocationFrom("../../resources.cfg");
    application.addResourceLocationFrom("./resources.cfg");
    application.create("../../plugins.cfg","../ogre.cfg","ogre.log",ST_GENERIC);
    application.run();
  }
  catch (Exception& e)
  {
    ErrorDialog* dlg = PlatformManager::getSingleton().createErrorDialog();
    dlg->display(e.getFullDescription());
    delete dlg;
    return 1;
  }
  return 0;
}

